
--[[

    
    eeSpeak reacts to various engineering events using emotes and quotes!
    Copyright (C) 2014 Lilih @ Defias Brotherhood (EU)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    
--]]

eeSpeak = {}

local L = LibStub("AceLocale-3.0"):GetLocale("eeSpeak")
eeSpeak = LibStub("AceAddon-3.0"):NewAddon("eeSpeak", 'AceEvent-3.0', 'AceConsole-3.0')

BINDING_HEADER_EESPEAK = "eeSpeak";
BINDING_NAME_ESOPTIONS = "Options";
BINDING_NAME_ESRELOADGFX = "Restart GFX engine";
BINDING_NAME_ESRELOADUI = "Reload UI";

local eeSpeak = eeSpeak
local dropDownMenu, db
local targetClass, englishClass
local genderTable
local BTspell = GetSpellInfo(126459)
--local BTspell = GetSpellInfo(5697)  -- debug unending breath spell
--local BTspell = GetSpellInfo(16589)  -- debug noggenfogger elixir spell
local oFRBspell = GetSpellInfo(22700)
local FRBspell = GetSpellInfo(44389)
local JVspell = GetSpellInfo(67826)
--local JVspell = GetSpellInfo(5697)  -- debug unending breath spell
--local FRBspell = GetSpellInfo(16589)  -- debug noggenfogger elixir spell
local EVEspell = GetSpellInfo(23442) -- goblin only
local GADspell = GetSpellInfo(23453) -- gnomish only
local A52spell = GetSpellInfo(36890) -- goblin only
local TOSspell = GetSpellInfo(36941) -- gnomish only
local NORspell = GetSpellInfo(67833)
local PANspell = GetSpellInfo(126755)

local PortalStyles = {
	"|cff00ff00"..L.PTBLINK, "|cffffd200"..L.PTSMILE, "|cffff6b00"..L.PTGOING, "|cffff2f32"..L.PTYAWN,
	"|cfff58cba"..L.PTWAVE, "|cffffdbad"..L.PTFAREWELL, "|cff7777aa"..L.PTTIRED, "|cff00ff00"..L.PTSMIRK,
	"|cffffd200"..L.PTPRAISE, "|cffff6b00"..L.PTLOVE,	"|cffff2f32"..L.PTLOST, "|cfff58cba"..L.PTKISS,
	"|cffffdbad"..L.PTGAZE,	"|cff7777aa"..L.PTABSENT,
}
local PortalEmotes = {
	"BLINK", "SMILE", "GOING", "YAWN", "WAVE", "BYE", "TIRED",
	"SMIRK", "PRAISE", "LOVE", "LOST", "KISS", "GAZE", "ABSENT",
}

local PartyChannels = {
	('PARTY'), ('SAY'), ('YELL'),
}
local PartyQuoteChannels = {
	"|cffaaeeff"..L.PARTY, L.SAY, "|cffff2f32"..L.YELL,
}
local RaidChannels = {
	('PARTY'), ('RAID'), ('SAY'), ('YELL'),
}
local RaidQuoteChannels = {
	"|cffaaeeff"..L.PARTY, "|cffff6b00"..L.RAID, L.SAY, "|cffff2f32"..L.YELL,
}
local SoloChannels = {
	('CHAT'), ('SAY'), ('YELL'),
}
local SoloQuoteChannels = {
	"|cffffdbad"..L.CHAT, L.SAY, "|cffff2f32"..L.YELL,
}

local toggle_option = { 
	name = function(info) return eeSpeak:GetName(info) end,
	desc = function(info)
	if string.find(info[#info], L.BLINGTRON) then return db.BlingtronQuotes[tonumber(string.match(info[#info], "%d%d"))]
	elseif string.find(info[#info], L.REPAIRBOT) then return db.RepairBotQuotes[tonumber(string.match(info[#info], "%d%d"))] end
	end,
	type = "input", get = "getESopt", set = "setESopt", disabled = "disESopt",
}

local eeSpeakBlock = LibStub("LibDataBroker-1.1"):NewDataObject("eeSpeak", {
	type = "data source", icon = "Interface\\Icons\\Trade_engineering",
	OnClick = function(self, button)
		if button == "LeftButton" and IsAltKeyDown() then
				RestartGx()
		elseif button == "RightButton" then
			if IsAltKeyDown() then
				ReloadUI()
			else
				eeSpeak:OpenOptions()
			end
		end
	end,
	OnTooltipShow = function(tooltip)
		local color = "|cffff0000"
		tooltip:AddLine("\124TInterface\\Icons\\Trade_engineering:14\124t eeSpeak v"..GetAddOnMetadata("eeSpeak", "version"))
		tooltip:AddLine(" ")

		color = "|cffff0000"
		if db.Blingtron == true then
			if db.BlingtronFrequency == 100 then color = "|cff00ff00"
			elseif db.BlingtronFrequency >= 75 and db.BlingtronFrequency < 100 then color = "|cff7fff7f"
			elseif db.BlingtronFrequency >= 50 and db.BlingtronFrequency < 75 then color = "|cffffff00"
			elseif db.BlingtronFrequency >= 25 and db.BlingtronFrequency < 50 then color = "|cffffff78"
			elseif db.BlingtronFrequency >= 1 and db.BlingtronFrequency < 25 then color = "|cffff6b00"
			end
			local tt = ""
			if db.BlingtronParty == true and db.BlingtronRaid == false and db.BlingtronSolo == false then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.BlingtronPartyChannels.Channel].."|cffffffff)]"
			elseif db.BlingtronParty == false and db.BlingtronRaid == true and db.BlingtronSolo == false then
				tt = tt.."[|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.BlingtronRaidChannels.Channel].."|cffffffff)]"
			elseif db.BlingtronParty == false and db.BlingtronRaid == false and db.BlingtronSolo == true then
				tt = tt.."[|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.BlingtronSoloChannels.Channel].."|cffffffff)]"
			elseif db.BlingtronParty == true and db.BlingtronRaid == true and db.BlingtronSolo == false then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.BlingtronPartyChannels.Channel].."|cffffffff)/|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.BlingtronRaidChannels.Channel].."|cffffffff)]"
			elseif db.BlingtronParty == true and db.BlingtronRaid == false and db.BlingtronSolo == true then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.BlingtronPartyChannels.Channel].."|cffffffff)/|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.BlingtronSoloChannels.Channel].."|cffffffff)]"
			elseif db.BlingtronParty == false and db.BlingtronRaid == true and db.BlingtronSolo == true then
				tt = tt.."[|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.BlingtronRaidChannels.Channel].."|cffffffff)/|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.BlingtronSoloChannels.Channel].."|cffffffff)]"
			elseif db.BlingtronParty == true and db.BlingtronRaid == true and db.BlingtronSolo == true then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.BlingtronPartyChannels.Channel].."|cffffffff)/|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.BlingtronRaidChannels.Channel].."|cffffffff)/|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.BlingtronSoloChannels.Channel].."|cffffffff)]"
			end
			tt = tt.."|cffffffff["..color..db.BlingtronFrequency.."%|cffffffff]"
			tooltip:AddDoubleLine("\124TInterface\\Icons\\Inv_pet_lilsmoky:14\124t |cffffffff"..L.BLINGTRON.." "..L.QUOTE,tt)
		else
			tooltip:AddDoubleLine("\124TInterface\\Icons\\Inv_pet_lilsmoky:14\124t |cffffffff"..L.BLINGTRON.." "..L.QUOTE,"|cffffffff["..color..L.OFF.."|cffffffff]")
		end

		color = "|cffff0000"
		if db.RepairBot == true then
			if db.RepairBotFrequency == 100 then color = "|cff00ff00"
			elseif db.RepairBotFrequency >= 75 and db.RepairBotFrequency < 100 then color = "|cff7fff7f"
			elseif db.RepairBotFrequency >= 50 and db.RepairBotFrequency < 75 then color = "|cffffff00"
			elseif db.RepairBotFrequency >= 25 and db.RepairBotFrequency < 50 then color = "|cffffff78"
			elseif db.RepairBotFrequency >= 1 and db.RepairBotFrequency < 25 then color = "|cffff6b00"
			end
			local tt = ""
			if db.RepairBotParty == true and db.RepairBotRaid == false and db.RepairBotSolo == false then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.RepairBotPartyChannels.Channel].."|cffffffff)]"
			elseif db.RepairBotParty == false and db.RepairBotRaid == true and db.RepairBotSolo == false then
				tt = tt.."[|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.RepairBotRaidChannels.Channel].."|cffffffff)]"
			elseif db.RepairBotParty == false and db.RepairBotRaid == false and db.RepairBotSolo == true then
				tt = tt.."[|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.RepairBotSoloChannels.Channel].."|cffffffff)]"
			elseif db.RepairBotParty == true and db.RepairBotRaid == true and db.RepairBotSolo == false then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.RepairBotPartyChannels.Channel].."|cffffffff)/|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.RepairBotRaidChannels.Channel].."|cffffffff)]"
			elseif db.RepairBotParty == true and db.RepairBotRaid == false and db.RepairBotSolo == true then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.RepairBotPartyChannels.Channel].."|cffffffff)/|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.RepairBotSoloChannels.Channel].."|cffffffff)]"
			elseif db.RepairBotParty == false and db.RepairBotRaid == true and db.RepairBotSolo == true then
				tt = tt.."[|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.RepairBotRaidChannels.Channel].."|cffffffff)/|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.RepairBotSoloChannels.Channel].."|cffffffff)]"
			elseif db.RepairBotParty == true and db.RepairBotRaid == true and db.RepairBotSolo == true then
				tt = tt.."[|cffaaeeff"..L.PARTY.."|cffffffff("..PartyQuoteChannels[db.RepairBotPartyChannels.Channel].."|cffffffff)/|cffff6b00"..L.RAID.."|cffffffff("..RaidQuoteChannels[db.RepairBotRaidChannels.Channel].."|cffffffff)/|cffffdbad"..L.SOLO.."|cffffffff("..SoloQuoteChannels[db.RepairBotSoloChannels.Channel].."|cffffffff)]"
			end
			tt = tt.."|cffffffff["..color..db.RepairBotFrequency.."%|cffffffff]"
			tooltip:AddDoubleLine("\124TInterface\\Icons\\inv_misc_enggizmos_01:14\124t |cffffffff"..L.REPAIRBOT.." "..L.QUOTE,tt)
		else
			tooltip:AddDoubleLine("\124TInterface\\Icons\\inv_misc_enggizmos_01:14\124t |cffffffff"..L.REPAIRBOT.." "..L.QUOTE,"|cffffffff["..color..L.OFF.."|cffffffff]")
		end

		tooltip:AddLine(" ")

		if db.Portal == true then
			if db.PortalFrequency == 100 then color = "|cff00ff00"
			elseif db.PortalFrequency >= 75 and db.PortalFrequency < 100 then color = "|cff7fff7f"
			elseif db.PortalFrequency >= 50 and db.PortalFrequency < 75 then color = "|cffffff00"
			elseif db.PortalFrequency >= 25 and db.PortalFrequency < 50 then color = "|cffffff78"
			elseif db.PortalFrequency >= 1 and db.PortalFrequency < 25 then color = "|cffff6b00"
			end
			local tt = ""
			if db.morningPortal == true then tt = tt.."[|cffADFF2F"..L.MORNING.."|cffffffff]" end
			if db.afternoonPortal == true then tt = tt.."[|cffADFF2F"..L.AFTERNOON.."|cffffffff]" end
			if db.eveningPortal == true then tt = tt.."[|cffADFF2F"..L.EVENING.."|cffffffff]" end
			if db.nightPortal == true then tt = tt.."[|cffADFF2F"..L.NIGHT.."|cffffffff]" end
			if db.randomPortal == true then 
				tt = tt.."[|cffFFC125"..L.RANDOM:gsub("^%l", string.upper).."|cffffffff]"
			else
				tt = tt.."[|cffff6060Fixed|cffffffff]"
			end
			tt = tt.."|cffffffff["..color..db.PortalFrequency.."%|cffffffff]"
			tooltip:AddDoubleLine("\124TInterface\\Icons\\Inv_misc_enggizmos_12:14\124t |cffffffff"..L.PORTAL.." "..L.EMOTE,tt)			
		else
			tooltip:AddDoubleLine("\124TInterface\\Icons\\Inv_misc_enggizmos_12:14\124t |cffffffff"..L.PORTAL.." "..L.EMOTE,"|cffffffff["..color..L.OFF.."|cffffffff]")
		end

		tooltip:AddLine(" ")
		
		color = "|cffff0000"
		if db.BlingtronGuild == true and db.Blingtron == true then
			color = "|cff00ff00"
			tooltip:AddDoubleLine("\124TInterface\\Icons\\achievement_guildperk_honorablemention:14\124t |cffffffff"..L.ANNOUNCE.." "..L.BLINGTRON.." "..L.TO.." "..L.GUILD,"|cffffffff["..color..L.ON.."|cffffffff]")
		elseif db.BlingtronGuild == false and db.Blingtron == true then
			tooltip:AddDoubleLine("\124TInterface\\Icons\\achievement_guildperk_honorablemention:14\124t |cffffffff"..L.ANNOUNCE.." "..L.BLINGTRON.." "..L.TO.." "..L.GUILD,"|cffffffff["..color..L.OFF.."|cffffffff]")
		end

		color = "|cffff0000"
		if db.BlingtronMapCoords == true and db.Blingtron == true then
			color = "|cff00ff00"
			tooltip:AddDoubleLine("\124TInterface\\Icons\\inv_misc_pocketwatch_02:14\124t |cffffffff"..L.ANNOUNCE.." "..L.BLINGTRON.." "..L.MAP,"|cffffffff["..color..L.ON.."|cffffffff]")
		elseif db.BlingtronMapCoords == false and db.Blingtron == true then
			tooltip:AddDoubleLine("\124TInterface\\Icons\\inv_misc_pocketwatch_02:14\124t |cffffffff"..L.ANNOUNCE.." "..L.BLINGTRON.." "..L.MAP,"|cffffffff["..color..L.OFF.."|cffffffff]")
		end		

		color = "|cffff0000"
		if db.RepairBotMapCoords == true and db.RepairBot == true then
			color = "|cff00ff00"
			tooltip:AddDoubleLine("\124TInterface\\Icons\\inv_misc_pocketwatch_02:14\124t |cffffffff"..L.ANNOUNCE.." "..L.REPAIRBOT.." "..L.MAP,"|cffffffff["..color..L.ON.."|cffffffff]")
		elseif db.RepairBotMapCoords == false and db.RepairBot == true then
			tooltip:AddDoubleLine("\124TInterface\\Icons\\inv_misc_pocketwatch_02:14\124t |cffffffff"..L.ANNOUNCE.." "..L.REPAIRBOT.." "..L.MAP,"|cffffffff["..color..L.OFF.."|cffffffff]")
		end				
		
		color = "|cffff0000"
		tooltip:AddLine(" ")
		tooltip:AddLine("|cffeda55f"..L.RIGHTCLICK.."|r "..L.FOROPTIONS, 0.2, 1, 0.2)
		tooltip:AddLine("|cffeda55f"..L.ALTCLICK.."|r "..L.RESTARTGFX, 0.2, 1, 0.2)
		tooltip:AddLine("|cffeda55f"..L.ALTRIGHTCLICK.."|r "..L.RELOADUI, 0.2, 1, 0.2)
	end,
})

local PortalOptions = {
	handler = eeSpeak,
	type = 'group',
	name = "\124TInterface\\Icons\\Inv_misc_enggizmos_12:16\124t "..L.PORTAL.." "..L.MANAGEMENT,
	order = 0,
	args = {
		header = {
			type = "header",
			name = "",
			order = 0,
		},
  	Portal = {
   		type = "toggle",
   		name = L.USE,
   		desc = L.QUOTESCASTING,
   		order = 1,
     	get = "getESopt",
     	set = "setESopt",
  	},
		PortalFrequency = {
			order = 2,
			type = "range",
			name = L.FREQUENCY,
			desc = L.FREQDESC,
			min = 1, max = 100, step = 1,
    	get = "getESopt",
    	set = "setESopt",
			disabled = "disESopt",
		},
 		ptTab = {
			type = "description",
			name = "",
			order = 3,
 		},
  	morningPortal = {
  		type = "toggle",
    	name = L.PTMORNING,
    	order = 5,
     	get = "getESopt",
    	set = "setESopt",
     	disabled = "disESopt",
  	},      								
		morningEmotes = {
			type = 'select',
			name = L.SELECTEDEMOTE,
			desc = L.SELECTMORNING,
			order = 6,
     	get = "getESopt",
    	set = "setESopt",
			values = PortalStyles,
     	disabled = "disESopt",
     	width = "double"
		},
  	afternoonPortal = {
  		type = "toggle",
    	name = L.PTAFTERNOON,
    	order = 8,
     	get = "getESopt",
    	set = "setESopt",
     	disabled = "disESopt",
  	},      								
		afternoonEmotes = {
			type = 'select',
			name = L.SELECTEDEMOTE,
			desc = L.SELECTAFTERNOON,
			order = 9,
     	get = "getESopt",
    	set = "setESopt",
			values = PortalStyles,
     	disabled = "disESopt",
     	width = "double"
		},
  	eveningPortal = {
  		type = "toggle",
    	name = L.PTEVENING,
    	order = 11,
     	get = "getESopt",
    	set = "setESopt",
     	disabled = "disESopt",
  	},      								
		eveningEmotes = {
			type = 'select',
			name = L.SELECTEDEMOTE,
			desc = L.SELECTEVENING,
			order = 12,
     	get = "getESopt",
    	set = "setESopt",
			values = PortalStyles,
     	disabled = "disESopt",
     	width = "double"
		},
    nightPortal = {
    	type = "toggle",
    	name = L.PTNIGHT,
    	order = 14,
     	get = "getESopt",
    	set = "setESopt",
     	disabled = "disESopt",
    },      								
		nightEmotes = {
			type = 'select',
			name = L.SELECTEDEMOTE,
			desc = L.SELECTNIGHT,
			order = 15,
     	get = "getESopt",
    	set = "setESopt",
			values = PortalStyles,
     	disabled = "disESopt",
     	width = "double"
		},
    randomPortal = {
    	type = "toggle",
    	name = L.USE.." "..L.RANDOM.." "..L.EMOTE,
    	order = 16,
     	get = "getESopt",
			set = "setESopt",
     	disabled = "disESopt",
    },
 	},
}

local BlingtronOptions = {
	handler = eeSpeak,
	type = 'group',
	childGroups = "tab",
	name = "\124TInterface\\Icons\\Inv_pet_lilsmoky:16\124t "..L.BLINGTRON.." 4000 "..L.MANAGEMENT,
	order = 1,
	args = {
		options = {
			name = "\124TInterface\\Icons\\Inv_pet_lilsmoky:16\124t "..L.OPTIONS,
			order = 0,
			type = "group",
			args = {
				Blingtron = {
 					type = "toggle",
   				name = L.USE,
   				desc = L.QUOTESCASTING,
					order = 1,
     			get = "getESopt",
     			set = "setESopt",
				},
				BlingtronFrequency = {
					order = 2,
				type = "range",
				name = L.FREQUENCY,
				desc = L.FREQDESC,
				min = 1, max = 100, step = 1,
   			get = "getESopt",
   			set = "setESopt",
				disabled = "disESopt",
				},
		 		btTab1 = {
					type = "description",
					name = "",
					order = 3,
 				},
 				BlingtronParty = {
 					type = "toggle",
   				name = L.INPARTY,
   				desc = L.USEQUOTESPARTY,
					order = 5,
     			get = "getESopt",
   				set = "setESopt",
					disabled = "disESopt",
 				},
				BlingtronPartyChannels = {
					type = 'select',
					name = L.SELECTEDCHANNEL,
					desc = L.SELECTCHANNEL,
					order = 6,
   				get = "getESopt",
   				set = "setESopt",
					values = PartyQuoteChannels,
					disabled = "disESopt",
					width = "double"
				},
 				BlingtronRaid = {
 					type = "toggle",
   				name = L.INRAID,
   				desc = L.USEQUOTESRAID,
					order = 8,
     			get = "getESopt",
   				set = "setESopt",
					disabled = "disESopt",
 				},
				BlingtronRaidChannels = {
					type = 'select',
					name = L.SELECTEDCHANNEL,
					desc = L.SELECTCHANNEL,
					order = 9,
   				get = "getESopt",
   				set = "setESopt",
					values = RaidQuoteChannels,
					disabled = "disESopt",
					width = "double"
				},
 				BlingtronSolo = {
 					type = "toggle",
   				name = L.INSOLO,
   				desc = L.USEQUOTESOLO,
					order = 10,
     			get = "getESopt",
   				set = "setESopt",
					disabled = "disESopt",
 				},
				BlingtronSoloChannels = {
					type = 'select',
					name = L.SELECTEDCHANNEL,
					desc = L.SELECTCHANNEL,
					order = 11,
   				get = "getESopt",
   				set = "setESopt",
					values = SoloQuoteChannels,
					disabled = "disESopt",
					width = "double"
				},
    		BlingtronGuild = {
    			type = "toggle",
    			name = L.ANNOUNCE.." "..L.TO.." "..L.GUILD,
    			order = 12,
     			get = "getESopt",
					set = "setESopt",
     			disabled = "disESopt",
    		},
		 		btTab2 = {
					type = "description",
					name = "",
					order = 13,
 				},
    		BlingtronMapCoords = {
    			type = "toggle",
    			name = L.ANNOUNCE.." "..L.MAP,
    			order = 14,
     			get = "getESopt",
					set = "setESopt",
     			disabled = "disESopt",
    		},
 			},
 		},
		BlingtronQuoteOptions = {
			type = 'group',
			name = "\124TInterface\\Icons\\Inv_misc_book_09:16\124t "..L.QUOTES,
			order = 10,
			args = {
				BlingtronQuotesmaxScale = {
					order = 14,
					type = "range",
					name = L.ENABLEDQUOTES,			
					desc = L.MAXTOUSE,
					min = 1, max = 18, step = 1,
 					get = "getESopt",
 					set = "setESopt",
 					disabled = "disESopt",
				},					
				BlingtronQuoteTab = {
					type = "description",
					name = function() local lq = "none" if db.lastBTquote ~= "" then lq = db.BlingtronQuotes[db.lastBTquote] end return "|cFF00FF00"..L.SUPPORT.."\n|cff7fff7f"..L.TAGDESCBLINGTRON.."\n|cFF00FF00"..L.GENPLAYER.." |cffff0000"..L.ONLY.." |cFF00FF00"..L.AND.." |cffff0000"..L.NOT.." |cFF00FF00"..L.CASESENSITIVE.."\n|cFF00FF00"..L.LAST.." "..L.QUOTE.."; |cffff0000<|r"..lq.."|cffff0000>" end,
					desc = " ",
					order = 15,
				},
				BlingtronquoteGroup = {
					type = "group",
					name = " ",
					guiInline = true,
					order = 16,
					childGroups = "input",
					args = {
						[L.BLINGTRON.." "..L.QUOTE.." #01"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #02"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #03"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #04"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #05"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #06"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #07"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #08"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #09"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #10"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #11"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #12"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #13"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #14"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #15"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #16"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #17"] = toggle_option,
						[L.BLINGTRON.." "..L.QUOTE.." #18"] = toggle_option,
					},
				},
			},
		},
	},
}

local RepairBotOptions = {
	handler = eeSpeak,
	type = 'group',
	childGroups = "tab",
	name = "\124TInterface\\Icons\\inv_misc_enggizmos_01:16\124t "..L.REPAIRBOT.." "..L.MANAGEMENT,
	order = 2,
	args = {
		options = {
			name = "\124TInterface\\Icons\\inv_misc_enggizmos_01:16\124t "..L.OPTIONS,
			order = 0,
			type = "group",
			args = {
				RepairBot = {
 					type = "toggle",
   				name = L.USE,
   				desc = L.QUOTESCASTING,
					order = 1,
     			get = "getESopt",
     			set = "setESopt",
				},
				RepairBotFrequency = {
					order = 2,
				type = "range",
				name = L.FREQUENCY,
				desc = L.FREQDESC,
				min = 1, max = 100, step = 1,
   			get = "getESopt",
   			set = "setESopt",
				disabled = "disESopt",
				},
		 		rbTab1 = {
					type = "description",
					name = "",
					order = 3,
 				},
 				RepairBotParty = {
 					type = "toggle",
   				name = L.INPARTY,
   				desc = L.USEQUOTESPARTY,
					order = 5,
     			get = "getESopt",
   				set = "setESopt",
					disabled = "disESopt",
 				},
				RepairBotPartyChannels = {
					type = 'select',
					name = L.SELECTEDCHANNEL,
					desc = L.SELECTCHANNEL,
					order = 6,
   				get = "getESopt",
   				set = "setESopt",
					values = PartyQuoteChannels,
					disabled = "disESopt",
					width = "double"
				},
 				RepairBotRaid = {
 					type = "toggle",
   				name = L.INRAID,
   				desc = L.USEQUOTESRAID,
					order = 8,
     			get = "getESopt",
   				set = "setESopt",
					disabled = "disESopt",
 				},
				RepairBotRaidChannels = {
					type = 'select',
					name = L.SELECTEDCHANNEL,
					desc = L.SELECTCHANNEL,
					order = 9,
   				get = "getESopt",
   				set = "setESopt",
					values = RaidQuoteChannels,
					disabled = "disESopt",
					width = "double"
				},
 				RepairBotSolo = {
 					type = "toggle",
   				name = L.INSOLO,
   				desc = L.USEQUOTESOLO,
					order = 10,
     			get = "getESopt",
   				set = "setESopt",
					disabled = "disESopt",
 				},
				RepairBotSoloChannels = {
					type = 'select',
					name = L.SELECTEDCHANNEL,
					desc = L.SELECTCHANNEL,
					order = 11,
   				get = "getESopt",
   				set = "setESopt",
					values = SoloQuoteChannels,
					disabled = "disESopt",
					width = "double"
				},
    		RepairBotMapCoords = {
    			type = "toggle",
    			name = L.ANNOUNCE.." "..L.MAP,
    			order = 14,
     			get = "getESopt",
					set = "setESopt",
     			disabled = "disESopt",
    		},
 			},
 		},
		RepairBotQuoteOptions = {
			type = 'group',
			name = "\124TInterface\\Icons\\Inv_misc_book_09:16\124t "..L.QUOTES,
			order = 10,
			args = {
				RepairBotQuotesmaxScale = {
					order = 14,
					type = "range",
					name = L.ENABLEDQUOTES,			
					desc = L.MAXTOUSE,
					min = 1, max = 18, step = 1,
 					get = "getESopt",
 					set = "setESopt",
 					disabled = "disESopt",
				},					
				RepairBotQuoteTab = {
					type = "description",
					name = function() local lq = "none" if db.lastRBquote ~= "" then lq = db.RepairbotQuotes[db.lastRBquote] end return "|cFF00FF00"..L.SUPPORT.."\n|cff7fff7f"..L.TAGDESCREPAIRBOT.."\n|cFF00FF00"..L.GENPLAYER.." |cffff0000"..L.ONLY.." |cFF00FF00"..L.AND.." |cffff0000"..L.NOT.." |cFF00FF00"..L.CASESENSITIVE.."\n|cFF00FF00"..L.LAST.." "..L.QUOTE.."; |cffff0000<|r"..lq.."|cffff0000>" end,
					desc = " ",
					order = 15,
				},
				RepairBotquoteGroup = {
					type = "group",
					name = " ",
					guiInline = true,
					order = 16,
					childGroups = "input",
					args = {
						[L.REPAIRBOT.." "..L.QUOTE.." #01"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #02"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #03"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #04"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #05"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #06"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #07"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #08"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #09"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #10"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #11"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #12"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #13"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #14"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #15"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #16"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #17"] = toggle_option,
						[L.REPAIRBOT.." "..L.QUOTE.." #18"] = toggle_option,
					},
				},
			},
		},
	},
}

local AboutFrame = {
	type = "group",
	name = L.ABOUT,
  desc = L.CLICKREAD,			
	order = 3,
	args = {
		header = {
			type = "header",
			name = L.ABOUT,
			order = 0,
		},
		logo = {
			type = "description",
			name = "\124TInterface\\Icons\\Trade_engineering:64\124t",
			order = 1,
		},				
		logdesc = {
			type = "description",
			name = L["ABOUT_TEXT"](GetAddOnMetadata("eeSpeak", "version")),
			order = 2,
		},								
	},
}

local defaults = {
	profile =  {
		lastSpell = "",
		
    Portal = true,
    morningPortal = true,
    afternoonPortal = true,
    eveningPortal = true,
    nightPortal = true,
    randomPortal = true,
    morningEmotes = { Emote = 1 },
    afternoonEmotes = { Emote = 2 },
    eveningEmotes = { Emote = 3 },
    nightEmotes = { Emote = 4 },
    PortalFrequency = 100,

    Blingtron = true,
    BlingtronParty = true,
    BlingtronRaid = true,
    BlingtronSolo = true,
    BlingtronGuild = true,
    BlingtronMapCoords = true,
    BlingtronPartyChannels = { Channel = 3 },
    BlingtronRaidChannels = { Channel = 4 },
    BlingtronSoloChannels = { Channel = 3 },
    BlingtronQuotesmaxScale = 10,
    BlingtronFrequency = 100,
    lastBTquote = "",

    BlingtronQuotes = {
			L.BLINGTRON1,
			L.BLINGTRON2,
			L.BLINGTRON3,
			L.BLINGTRON4,
			L.BLINGTRON5,
			L.BLINGTRON6,
			L.BLINGTRON7,
			L.BLINGTRON8,
			L.BLINGTRON9,																					
			L.BLINGTRON10,
			L.BLINGTRON11,
			L.BLINGTRON12,
			L.BLINGTRON13,
			L.BLINGTRON14,
			L.BLINGTRON15,
			L.BLINGTRON16,
			L.BLINGTRON17,
			L.BLINGTRON18,
    },
    RepairBot = true,
    RepairBotParty = true,
    RepairBotRaid = true,
    RepairBotSolo = true,
    RepairBotMapCoords = true,
    RepairBotPartyChannels = { Channel = 1 },
    RepairBotRaidChannels = { Channel = 2 },
    RepairBotSoloChannels = { Channel = 2 },
    RepairBotQuotesmaxScale = 1,
    RepairBotFrequency = 100,
    lastRBquote = "",

    RepairBotQuotes = {
			L.REPAIRBOT1,
			L.REPAIRBOT2,
			L.REPAIRBOT3,
			L.REPAIRBOT4,
			L.REPAIRBOT5,
			L.REPAIRBOT6,
			L.REPAIRBOT7,
			L.REPAIRBOT8,
			L.REPAIRBOT9,																					
			L.REPAIRBOT10,
			L.REPAIRBOT11,
			L.REPAIRBOT12,
			L.REPAIRBOT13,
			L.REPAIRBOT14,
			L.REPAIRBOT15,
			L.REPAIRBOT16,
			L.REPAIRBOT17,
			L.REPAIRBOT18,
    },
	},
}

local function txtReplace(txt)
	local playerClass, englishClass = UnitClass("player")
	txt = txt:gsub(L.PLAYER, UnitName("player"))	
	if db.lastSpell == oFRBspell then
		txt = txt:gsub(L.BOT, "\124cffffffff\124Hitem:18232:0:0:0:0:0:0:0:0:0:0\124h[Field Repair Bot 74A]\124h\124r")
	elseif db.lastSpell == FRBspell then
		txt = txt:gsub(L.BOT, "\124cffffffff\124Hitem:34113:0:0:0:0:0:0:0:0:0:0\124h[Field Repair Bot 110G]\124h\124r")
	elseif db.lastSpell == JVspell then
		txt = txt:gsub(L.BOT, "\124cffa335ee\124Hitem:49040:0:0:0:0:0:0:0:0:0:0\124h[Jeeves]\124h\124r")
	end
	SetMapToCurrentZone()
	local px,py=_G.GetPlayerMapPosition("player")
	if db.BlingtronMapCoords == true and db.lastSpell == BTspell then
		txt = txt:gsub(L.LOCATION, format("%s (%.1f, %.1f)",GetMinimapZoneText(),px*100,py*100))
	elseif db.RepairBotMapCoords == true and (db.lastSpell == oFRBspell or db.lastSpell == FRBspell or db.lastSpell == JVspell) then
		txt = txt:gsub(L.LOCATION, format("%s (%.1f, %.1f)",GetMinimapZoneText(),px*100,py*100))
	else
		txt = txt:gsub(L.LOCATION, GetMinimapZoneText())
	end
	local genderTable = { L.IT, L.HE, L.SHE }
	local gender = string.match(txt, genderTable[UnitSex("player")])
	if gender ~= nil then
		txt = txt:gsub(L.HESHE, gender)
		gender = nil
	end
	genderTable = { L.ITS, L.HIM, L.HER }	
	gender = string.match(txt, genderTable[UnitSex("player")])
	if gender ~= nil then
		txt = txt:gsub(L.HIMHER, gender)
		gender = nil
	end
	genderTable = { L.ITS, L.HIS, L.HER }	
	gender = string.match(txt, genderTable[UnitSex("player")])
	if gender ~= nil then
		txt = txt:gsub(L.HISHER, gender)
		gender = nil
	end
	genderTable = { L.ITS, L.HIS, L.HERS }	
	gender = string.match(txt, genderTable[UnitSex("player")])
	if gender ~= nil then
		txt = txt:gsub(L.HISHERS, gender)
		gender = nil
	end		
	if UnitName("target") ~= nil then
		local targetClass, englishClass = UnitClass("target")	
		txt = txt:gsub(L.TARGET, (UnitName("target")))
		txt = txt:gsub(L.TARGETCLASS, targetClass)
	else
		txt = txt:gsub(L.TARGET, L.NOTARGET)
		txt = txt:gsub(L.TARGETCLASS, L.NOTARGET)
	end		
	return txt
end

function eeSpeak:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("eeSpeakDB", defaults)
  db = self.db.profile
	eeSpeak.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	eeSpeak.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	eeSpeak.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")	
	local AceConfig = LibStub("AceConfig-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")
  local about_panel = LibStub:GetLibrary("LibAboutPanel", true)
  
	if about_panel then
		self.optionsFrame = about_panel.new(nil, "eeSpeak")
	end

  AceConfig:RegisterOptionsTable("Blingtron Options", BlingtronOptions)
  AceConfig:RegisterOptionsTable("RepairBot Options", RepairBotOptions)
  AceConfig:RegisterOptionsTable("Portal emotes", PortalOptions)
  AceConfig:RegisterOptionsTable("eeSpeak About", AboutFrame)
  AceConfig:RegisterOptionsTable("eeSpeak Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))

  AceConfigDialog:AddToBlizOptions("Blingtron Options", L.BLINGTRON, "eeSpeak")
  AceConfigDialog:AddToBlizOptions("RepairBot Options", L.REPAIRBOT, "eeSpeak")
  AceConfigDialog:AddToBlizOptions("Portal emotes", L.PORTAL, "eeSpeak")
  AceConfigDialog:AddToBlizOptions("eeSpeak About", L.ABOUT, "eeSpeak")
  AceConfigDialog:AddToBlizOptions("eeSpeak Profiles", L.PROFILES, "eeSpeak")

  self:RegisterChatCommand("ee", "ChatCommand")
  self:RegisterChatCommand("eeSpeak", "ChatCommand")
  self:Print("|cffffff78<eeSpeak>|r v|cffffd200"..GetAddOnMetadata("eeSpeak", "version").."|r initialized. Type /ee or use hotkey for options.")
end

function eeSpeak:OnProfileChanged(event, database, newProfileKey)
	self.db = LibStub("AceDB-3.0"):New("eeSpeakDB", defaults)
  db = self.db.profile
 	eeSpeak.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	eeSpeak.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	eeSpeak.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
end

function eeSpeak:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
end

function eeSpeak:GetName(info)
	local n = info[#info]
	return (n)
end

function eeSpeak:ChatCommand(input)
	PlaySound("igSpellBookOpen")
	-- Fix for Blizzard addon panel (http://www.wowace.com/paste/8364)
	-- Bail out if already loaded and up to date
	local MAJOR, MINOR = "InterfaceOptionsFix", 1
--	if _G[MAJOR] and _G[MAJOR].version >= MINOR then return end
	-- Reuse the existing frame or create a new one
	local frame = _G[MAJOR] or CreateFrame("Frame", MAJOR, _G.InterfaceOptionsFrame)
	frame.version = MINOR
	-- Hook once and the call the possibly upgraded methods
	if not frame.Saved_InterfaceOptionsFrame_OpenToCategory then
  	-- Save the unhooked function
    frame.Saved_InterfaceOptionsFrame_OpenToCategory = _G.InterfaceOptionsFrame_OpenToCategory
    hooksecurefunc("InterfaceOptionsFrame_OpenToCategory", function(...)
    	return frame:InterfaceOptionsFrame_OpenToCategory(...)
    end)
    -- Please note that the frame is a child of InterfaceOptionsFrame, so OnUpdate won't called before InterfaceOptionsFrame is shown
    frame:SetScript('OnUpdate', function(_, ...)
        return frame:OnUpdate(...)
    end)
	end
	-- This will be called twice on first open : 
	-- 1) with the panel which is actually wanted,
	-- 2) with the "Control" panel from InterfaceOptionsFrame_OnShow (this is what actually cause the bug). 
	function frame:InterfaceOptionsFrame_OpenToCategory(panel)
  	self.panel = panel
	end
	function frame:OnUpdate()
    local panel = self.panel   
    -- Clean up
    self:SetScript('OnUpdate', nil)
    self:Hide()
    self.panel = nil
    self.InterfaceOptionsFrame_OpenToCategory = function() end
    -- Call the original InterfaceOptionsFrame_OpenToCategory with the last panel
    self.Saved_InterfaceOptionsFrame_OpenToCategory(panel)
	end
	-- EoF
	InterfaceOptionsFrame_OpenToCategory(eeSpeak.optionsFrame)
end

function eeSpeak:OpenOptions()
	PlaySound("igSpellBookOpen")
	-- Fix for Blizzard addon panel (http://www.wowace.com/paste/8364)
	-- Bail out if already loaded and up to date
	local MAJOR, MINOR = "InterfaceOptionsFix", 1
--	if _G[MAJOR] and _G[MAJOR].version >= MINOR then return end
	-- Reuse the existing frame or create a new one
	local frame = _G[MAJOR] or CreateFrame("Frame", MAJOR, _G.InterfaceOptionsFrame)
	frame.version = MINOR
	-- Hook once and the call the possibly upgraded methods
	if not frame.Saved_InterfaceOptionsFrame_OpenToCategory then
  	-- Save the unhooked function
    frame.Saved_InterfaceOptionsFrame_OpenToCategory = _G.InterfaceOptionsFrame_OpenToCategory
    hooksecurefunc("InterfaceOptionsFrame_OpenToCategory", function(...)
    	return frame:InterfaceOptionsFrame_OpenToCategory(...)
    end)
    -- Please note that the frame is a child of InterfaceOptionsFrame, so OnUpdate won't called before InterfaceOptionsFrame is shown
    frame:SetScript('OnUpdate', function(_, ...)
        return frame:OnUpdate(...)
    end)
	end
	-- This will be called twice on first open : 
	-- 1) with the panel which is actually wanted,
	-- 2) with the "Control" panel from InterfaceOptionsFrame_OnShow (this is what actually cause the bug). 
	function frame:InterfaceOptionsFrame_OpenToCategory(panel)
  	self.panel = panel
	end
	function frame:OnUpdate()
    local panel = self.panel   
    -- Clean up
    self:SetScript('OnUpdate', nil)
    self:Hide()
    self.panel = nil
    self.InterfaceOptionsFrame_OpenToCategory = function() end
    -- Call the original InterfaceOptionsFrame_OpenToCategory with the last panel
    self.Saved_InterfaceOptionsFrame_OpenToCategory(panel)
	end
	-- EoF
	InterfaceOptionsFrame_OpenToCategory(eeSpeak.optionsFrame)
end

function eeSpeak:UNIT_SPELLCAST_SENT(arg1,arg2,arg3,arg4,arg5,arg6)
	if (arg3 == EVEspell or arg3 == GADspell or arg3 == A52spell or arg3 == TOSspell or arg3 == NORspell or arg3 == PANspell) and db.Portal == true and random(1,100) <= db.PortalFrequency then
		local hour,minute = GetGameTime()
		local second = (hour * 3600) + (minute * 60)
		if (second >= 0 and second < 21600) and db.nightPortal == true  then
			if db.randomPortal == true then DoEmote(PortalEmotes[random(1,14)])
			else
				DoEmote(PortalEmotes[db.nightEmotes.Emote])
			end
		elseif (second >= 21600 and second < 43200) and db.morningPortal == true then
			if db.randomPortal == true then
				DoEmote(PortalEmotes[random(1,14)])
			else
				DoEmote(PortalEmotes[db.morningEmotes.Emote])
			end
		elseif (second >= 43200 and second < 64800) and db.afternoonPortal == true then
			if db.randomPortal == true then
				DoEmote(PortalEmotes[random(1,14)])
			else
				DoEmote(PortalEmotes[db.afternoonEmotes.Emote])
			end
		elseif (second >= 64800 and second <= 86400) and db.eveningPortal == true then
			if db.randomPortal == true then
				DoEmote(PortalEmotes[random(1,14)])
			else
				DoEmote(PortalEmotes[db.eveningEmotes.Emote])
			end
		end
	end
	
	if db.Blingtron == true and arg2 == "player" and arg3 == BTspell and random(1,100) <= db.BlingtronFrequency then
		local i = 0		
		if db.BlingtronQuotesmaxScale == 1 then i = 1
		else
			repeat 
				i = random(1,db.BlingtronQuotesmaxScale)
  		until i ~= db.lastBTquote
		end
		db.lastBTquote = i		
		db.lastSpell = BTspell
		if db.BlingtronGuild == true and db.BlingtronQuotes[i] ~= "" then SendChatMessage(txtReplace(db.BlingtronQuotes[i]),"GUILD",nil,"GUILD") end
		if GetNumGroupMembers() > 5 and db.BlingtronRaid == true then
			if db.BlingtronQuotes[i] ~= "" then
				if (IsInGroup(LE_PARTY_CATEGORY_INSTANCE))
				and 
				RaidChannels[db.BlingtronRaidChannels.Channel] ~= "SAY"
				and 
				RaidChannels[db.BlingtronRaidChannels.Channel] ~= "YELL" then
					SendChatMessage(txtReplace(db.BlingtronQuotes[i]),"INSTANCE_CHAT")
				else	
					SendChatMessage(txtReplace(db.BlingtronQuotes[i]),RaidChannels[db.BlingtronRaidChannels.Channel])
				end
			else
				self:Print("|cffff0000"..L.WARNING.." |cffB0A0ff"..L.EMPTYQUOTE)				
			end
		elseif GetNumGroupMembers() > 0 and GetNumGroupMembers() < 6 and db.BlingtronParty == true then
			if db.BlingtronQuotes[i] ~= "" then
				if (IsInGroup(LE_PARTY_CATEGORY_INSTANCE))
				and 
				PartyChannels[db.BlingtronPartyChannels.Channel] ~= "SAY"
				and 
				PartyChannels[db.BlingtronPartyChannels.Channel] ~= "YELL" then
					SendChatMessage(txtReplace(db.BlingtronQuotes[i]),"INSTANCE_CHAT")
				else
				SendChatMessage(txtReplace(db.BlingtronQuotes[i]),PartyChannels[db.BlingtronPartyChannels.Channel])
				end
			else
				self:Print("|cffff0000"..L.WARNING.." |cffB0A0ff"..L.EMPTYQUOTE)				
			end			
		elseif GetNumGroupMembers() == 0 and db.BlingtronSolo == true then
			if db.BlingtronQuotes[i] ~= "" then
				if SoloChannels[db.BlingtronSoloChannels.Channel] ~= "SAY"
				and 
				SoloChannels[db.BlingtronSoloChannels.Channel] ~= "YELL" then
					self:Print("|cffff0000"..L.CHATONLY.." |r"..txtReplace(db.BlingtronQuotes[i]))
				else
				SendChatMessage(txtReplace(db.BlingtronQuotes[i]),SoloChannels[db.BlingtronSoloChannels.Channel])
				end
			else
				self:Print("|cffff0000"..L.WARNING.." |cffB0A0ff"..L.EMPTYQUOTE)				
			end			
		end
	end
	
	if db.RepairBot == true and arg2 == "player" and (arg3 == oFRBspell or arg3 == FRBspell or arg3 == JVspell) and random(1,100) <= db.RepairBotFrequency then
		db.lastSpell = arg3
		local i = 0		
		if db.RepairBotQuotesmaxScale == 1 then i = 1
		else
			repeat 
				i = random(1,db.RepairBotQuotesmaxScale)
  		until i ~= db.lastBTquote
		end
		db.lastBTquote = i
		if GetNumGroupMembers() > 5 and db.RepairBotRaid == true then
			if db.RepairBotQuotes[i] ~= "" then
				if (IsInGroup(LE_PARTY_CATEGORY_INSTANCE))
				and 
				RaidChannels[db.RepairBotRaidChannels.Channel] ~= "SAY"
				and 
				RaidChannels[db.RepairBotRaidChannels.Channel] ~= "YELL" then
					SendChatMessage(txtReplace(db.RepairBotQuotes[i]),"INSTANCE_CHAT")
				else	
					SendChatMessage(txtReplace(db.RepairBotQuotes[i]),RaidChannels[db.RepairBotRaidChannels.Channel])
				end
			else
				self:Print("|cffff0000"..L.WARNING.." |cffB0A0ff"..L.EMPTYQUOTE)				
			end
		elseif GetNumGroupMembers() > 0 and GetNumGroupMembers() < 6 and db.RepairBotParty == true then
			if db.RepairBotQuotes[i] ~= "" then
				if (IsInGroup(LE_PARTY_CATEGORY_INSTANCE))
				and 
				PartyChannels[db.RepairBotPartyChannels.Channel] ~= "SAY"
				and 
				PartyChannels[db.RepairBotPartyChannels.Channel] ~= "YELL" then
					SendChatMessage(txtReplace(db.RepairBotQuotes[i]),"INSTANCE_CHAT")
				else
				SendChatMessage(txtReplace(db.RepairBotQuotes[i]),PartyChannels[db.RepairBotPartyChannels.Channel])
				end
			else
				self:Print("|cffff0000"..L.WARNING.." |cffB0A0ff"..L.EMPTYQUOTE)				
			end			
		elseif GetNumGroupMembers() == 0 and db.RepairBotSolo == true then
			if db.RepairBotQuotes[i] ~= "" then
				if SoloChannels[db.RepairBotSoloChannels.Channel] ~= "SAY"
				and 
				SoloChannels[db.RepairBotSoloChannels.Channel] ~= "YELL" then
					self:Print("|cffff0000"..L.CHATONLY.." |r"..txtReplace(db.RepairBotQuotes[i]))
				else
				SendChatMessage(txtReplace(db.RepairBotQuotes[i]),SoloChannels[db.RepairBotSoloChannels.Channel])
				end
			else
				self:Print("|cffff0000"..L.WARNING.." |cffB0A0ff"..L.EMPTYQUOTE)				
			end			
		end
	end	
	db.lastSpell = ""
end

function eeSpeak:getESopt(info, ...)
	if string.find(info[#info], "Emotes") then return db[info[#info]].Emote
	elseif string.find(info[#info], "Channels") then return db[info[#info]].Channel
	elseif string.find(info[#info], L.BLINGTRON.." "..L.QUOTE) then return db.BlingtronQuotes[tonumber(string.match(info[#info], "%d%d"))]
	elseif string.find(info[#info], L.REPAIRBOT.." "..L.QUOTE) then return db.RepairBotQuotes[tonumber(string.match(info[#info], "%d%d"))]		
	else return db[info[#info]] end
end

function eeSpeak:setESopt(info, newValue)
	if string.find(info[#info], "Emotes") then db[info[#info]].Emote = newValue
	elseif string.find(info[#info], "Channels") then db[info[#info]].Channel = newValue
	elseif string.find(info[#info], L.BLINGTRON.." "..L.QUOTE) then db.BlingtronQuotes[tonumber(string.match(info[#info], "%d%d"))] = newValue
	elseif string.find(info[#info], L.REPAIRBOT.." "..L.QUOTE) then db.RepairBotQuotes[tonumber(string.match(info[#info], "%d%d"))] = newValue		

	elseif info[#info] == "morningPortal" then
		if db.Portal == true and newValue == false and db.afternoonPortal == false and db.eveningPortal == false and db.nightPortal == false then db.morningPortal = true
		elseif newValue == true then db.morningPortal = true
    else db.morningPortal = false end
 	elseif info[#info] == "afternoonPortal" then
 		if db.Portal == true and newValue == false and db.morningPortal == false and db.eveningPortal == false and db.nightPortal == false then db.afternoonPortal = true
		elseif newValue == true then db.afternoonPortal = true
    else db.afternoonPortal = false end
 	elseif info[#info] == "eveningPortal" then
 		if db.Portal == true and newValue == false and db.morningPortal == false and db.afternoonPortal == false and db.nightPortal == false then db.eveningPortal = true
		elseif newValue == true then db.eveningPortal = true
    else db.eveningPortal = false end
 	elseif info[#info] == "nightPortal" then
 		if db.Portal == true and newValue == false and db.morningPortal == false and db.afternoonPortal == false and db.eveningPortal == false then db.nightPortal = true
		elseif newValue == true then db.nightPortal = true
 		else db.nightPortal = false end

	elseif info[#info] == "BlingtronParty" then
		if newValue == false and db.BlingtronRaid == false and db.BlingtronSolo == false then db.BlingtronParty = true
		elseif newValue == true then db.BlingtronParty = true
		else db.BlingtronParty = false end
	elseif info[#info] == "BlingtronRaid" then
  	if newValue == false and db.BlingtronParty == false and db.BlingtronSolo == false then db.BlingtronRaid = true
		elseif newValue == true then db.BlingtronRaid = true
  	else db.BlingtronRaid = false end
	elseif info[#info] == "BlingtronSolo" then
		if newValue == false and db.BlingtronParty == false and db.BlingtronRaid == false then db.BlingtronSolo = true
		elseif newValue == true then db.BlingtronSolo = true
  	else db.BlingtronSolo = false end
  		
	elseif info[#info] == "RepairBotParty" then
		if newValue == false and db.RepairBotRaid == false and db.RepairBotSolo == false then db.RepairBotParty = true
		elseif newValue == true then db.RepairBotParty = true
		else db.RepairBotParty = false end
	elseif info[#info] == "RepairBotRaid" then
  	if newValue == false and db.RepairBotParty == false and db.RepairBotSolo == false then db.RepairBotRaid = true
		elseif newValue == true then db.RepairBotRaid = true
  	else db.RepairBotRaid = false end
	elseif info[#info] == "RepairBotSolo" then
		if newValue == false and db.RepairBotParty == false and db.RepairBotRaid == false then db.RepairBotSolo = true
		elseif newValue == true then db.RepairBotSolo = true
  	else db.RepairBotSolo = false end  		
  	
	else
		db[info[#info]] = newValue
	end
end

function eeSpeak:disESopt(info, ...)
	if info[#info] == "morningEmotes" then return (db.Portal == false or db.morningPortal == false or db.randomPortal == true)
	elseif info[#info] == "afternoonEmotes" then return (db.Portal == false or db.afternoonPortal == false or db.randomPortal == true)
	elseif info[#info] == "eveningEmotes" then return (db.Portal == false or db.eveningPortal == false or db.randomPortal == true)
	elseif info[#info] == "nightEmotes" then return (db.Portal == false or db.nightPortal == false or db.randomPortal == true)
	elseif string.find(info[#info], L.PORTAL) then return (db.Portal == false)

	elseif string.find(info[#info], L.BLINGTRON.." "..L.QUOTE) then return (db.Blingtron == false or (db.BlingtronQuoteFrequency == 0) or (db.BlingtronQuotesmaxScale < tonumber(string.match(info[#info], "%d%d")) ))
	elseif info[#info] == "BlingtronPartyChannels" then return (db.Blingtron == false or db.BlingtronParty == false)
	elseif info[#info] == "BlingtronRaidChannels" then return (db.Blingtron == false or db.BlingtronRaid == false)
	elseif string.find(info[#info], "Blingtron") then return (db.Blingtron == false)
		
	elseif string.find(info[#info], L.REPAIRBOT.." "..L.QUOTE) then return (db.RepairBot == false or (db.RepairBotQuoteFrequency == 0) or (db.RepairBotQuotesmaxScale < tonumber(string.match(info[#info], "%d%d")) ))
	elseif info[#info] == "RepairBotPartyChannels" then return (db.RepairBot == false or db.RepairBotParty == false)
	elseif info[#info] == "RepairBotRaidChannels" then return (db.RepairBot == false or db.RepairBotRaid == false)
	elseif string.find(info[#info], "RepairBot") then return (db.RepairBot == false)		

	end
end